@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, Trog, Török Edvin
@@ -49,7 +50,7 @@
 #include "shared/output.h"
 #include "shared/optparser.h"
 
-#include "fan.h"
+#include "onaccess_fan.h"
 #include "server.h"
 #include "thrmgr.h"
 #include "session.h"
@@ -296,6 +297,15 @@ static const char *get_cmd(struct fd_buf *buf, size_t off, size_t *len, char *te
     }
 }
 
+int statinidir_th(const char* dirname)
+{
+   if (!dbstat.entries) {
+      memset(&dbstat, 0, sizeof(dbstat));
+   }
+
+   return cl_statinidir(dirname, &dbstat);
+}
+
 struct acceptdata {
     struct fd_data fds;
     struct fd_data recv_fds;
@@ -874,6 +884,33 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     val = cl_engine_get_num(engine, CL_ENGINE_MAX_ICONSPE, NULL);
     logg("Limits: MaxIconsPE limit set to %llu.\n", val);
 
+    if((opt = optget(opts, "MaxRecHWP3"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_RECHWP3, opt->numarg))) {
+            logg("!cli_engine_set_num(MaxRecHWP3) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 1;
+        }
+    }
+    val = cl_engine_get_num(engine, CL_ENGINE_MAX_RECHWP3, NULL);
+    logg("Limits: MaxRecHWP3 limit set to %llu.\n", val);
+
+    /* options are handled in main (clamd.c) */
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_MATCH_LIMIT, NULL);
+    logg("Limits: PCREMatchLimit limit set to %llu.\n", val);
+
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_RECMATCH_LIMIT, NULL);
+    logg("Limits: PCRERecMatchLimit limit set to %llu.\n", val);
+
+    if((opt = optget(opts, "PCREMaxFileSize"))->active) {
+        if((ret = cl_engine_set_num(engine, CL_ENGINE_PCRE_MAX_FILESIZE, opt->numarg))) {
+            logg("!cli_engine_set_num(PCREMaxFileSize) failed: %s\n", cl_strerror(ret));
+            cl_engine_free(engine);
+            return 1;
+        }
+    }
+    val = cl_engine_get_num(engine, CL_ENGINE_PCRE_MAX_FILESIZE, NULL);
+    logg("Limits: PCREMaxFileSize limit set to %llu.\n", val);
+
     if(optget(opts, "ScanArchive")->enabled) {
 	logg("Archive support enabled.\n");
 	options |= CL_SCAN_ARCHIVE;
@@ -960,6 +997,20 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	logg("HTML support disabled.\n");
     }
 
+    if(optget(opts, "ScanXMLDOCS")->enabled) {
+	logg("XMLDOCS support enabled.\n");
+	options |= CL_SCAN_XMLDOCS;
+    } else {
+	logg("XMLDOCS support disabled.\n");
+    }
+
+    if(optget(opts, "ScanHWP3")->enabled) {
+	logg("HWP3 support enabled.\n");
+	options |= CL_SCAN_HWP3;
+    } else {
+	logg("HWP3 support disabled.\n");
+    }
+
     if(optget(opts,"PhishingScanURLs")->enabled) {
 
 	if(optget(opts,"PhishingAlwaysBlockCloak")->enabled) {
@@ -1024,7 +1075,6 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     } else {
 	logg("Self checking every %u seconds.\n", selfchk);
     }
-    memset(&dbstat, 0, sizeof(dbstat));
 
     /* save the PID */
     mainpid = getpid();
@@ -1102,6 +1152,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     acceptdata.max_queue = max_queue;
 
     if(optget(opts, "ScanOnAccess")->enabled)
+
 #if defined(FANOTIFY) || defined(CLAMAUTH)
     {
         do {
@@ -1111,7 +1162,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	    tharg->opts = opts;
 	    tharg->engine = engine;
 	    tharg->options = options;
-	    if(!pthread_create(&fan_pid, &fan_attr, fan_th, tharg)) break;
+	    if(!pthread_create(&fan_pid, &fan_attr, onas_fan_th, tharg)) break;
 	    free(tharg);
 	    tharg=NULL;
 	} while(0);
@@ -1121,6 +1172,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	logg("!On-access scan is not available\n");
 #endif
 
+
 #ifndef	_WIN32
     /* set up signal handling */
     sigfillset(&sigset);
@@ -1386,15 +1438,7 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	pthread_mutex_lock(&reload_mutex);
 	if(reload) {
 	    pthread_mutex_unlock(&reload_mutex);
-#if defined(FANOTIFY) || defined(CLAMAUTH)
-	    if(optget(opts, "ScanOnAccess")->enabled && tharg) {
-		logg("Restarting on-access scan\n");
-		pthread_mutex_lock(&logg_mutex);
-		pthread_kill(fan_pid, SIGUSR1);
-		pthread_mutex_unlock(&logg_mutex);
-		pthread_join(fan_pid, NULL);
-	    }
-#endif
+
 	    engine = reload_db(engine, dboptions, opts, FALSE, &ret);
 	    if(ret) {
 		logg("Terminating because of a fatal error.\n");
@@ -1407,10 +1451,10 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
 	    reload = 0;
 	    time(&reloaded_time);
 	    pthread_mutex_unlock(&reload_mutex);
+
 #if defined(FANOTIFY) || defined(CLAMAUTH)
 	    if(optget(opts, "ScanOnAccess")->enabled && tharg) {
 		tharg->engine = engine;
-		pthread_create(&fan_pid, &fan_attr, fan_th, tharg);
 	    }
 #endif
 	    time(&start_time);
@@ -1477,4 +1521,4 @@ int recvloop_th(int *socketds, unsigned nsockets, struct cl_engine *engine, unsi
     logg("--- Stopped at %s", cli_ctime(&current_time, timestr, sizeof(timestr)));
 
     return ret;
-}
+} 
